@@ -1,4 +1,4 @@
-// Copyright 2011 Google Inc.
+// Copyright 2011 Google Inc. All Rights Reserved.
 //
 // This code is licensed under the same terms as WebM:
 //  Software License Agreement:  http://www.webmproject.org/license/software/
@@ -27,6 +27,10 @@
 #include <jpeglib.h>
 #endif
 
+#ifdef WEBP_HAVE_TIFF
+#include <tiffio.h>
+#endif
+
 #ifdef HAVE_WINCODEC_H
 #ifdef __MINGW32__
 #define INITGUID  // Without this GUIDs are declared extern and fail to link
@@ -38,35 +42,35 @@
 #include <shlwapi.h>
 #include <windows.h>
 #include <wincodec.h>
-
-#ifndef GUID_WICPixelFormat24bppRGB
-// From Microsoft SDK 7.0a
-DEFINE_GUID(GUID_WICPixelFormat24bppRGB,
-    0x6fddc324, 0x4e03, 0x4bfe, 0xb1, 0x85, 0x3d, 0x77, 0x76, 0x8d, 0xc9, 0x0d);
-#endif
-#ifndef GUID_WICPixelFormat32bppRGBA
-DEFINE_GUID(GUID_WICPixelFormat32bppRGBA,
-    0xf5c7ad2d, 0x6a8d, 0x43dd, 0xa7, 0xa8, 0xa2, 0x99, 0x35, 0x26, 0x1a, 0xe9);
-#endif
 #endif  /* HAVE_WINCODEC_H */
 
 
 #include "webp/encode.h"
-#include "stopwatch.h"
+#include "./stopwatch.h"
 #ifndef WEBP_DLL
-extern void* VP8GetCPUInfo;   // opaque forward declaration.
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
 #endif
 
+extern void* VP8GetCPUInfo;   // opaque forward declaration.
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}    // extern "C"
+#endif
+#endif  // WEBP_DLL
+
 //------------------------------------------------------------------------------
 
 static int verbose = 0;
 
 static int ReadYUV(FILE* in_file, WebPPicture* const pic) {
+  const int use_argb = pic->use_argb;
   const int uv_width = (pic->width + 1) / 2;
   const int uv_height = (pic->height + 1) / 2;
   int y;
   int ok = 0;
 
+  pic->use_argb = 0;
   if (!WebPPictureAlloc(pic)) return ok;
 
   for (y = 0; y < pic->height; ++y) {
@@ -83,6 +87,7 @@ static int ReadYUV(FILE* in_file, WebPPicture* const pic) {
       goto End;
   }
   ok = 1;
+  if (use_argb) ok = WebPPictureYUVAToARGB(pic);
 
  End:
   return ok;
@@ -90,32 +95,65 @@ static int ReadYUV(FILE* in_file, WebPPicture* const pic) {
 
 #ifdef HAVE_WINCODEC_H
 
-#define IFS(fn)                \
-  do {                         \
-     if (SUCCEEDED(hr))        \
-     {                         \
-        hr = (fn);             \
-        if (FAILED(hr) && verbose)           \
-          printf(#fn " failed %08x\n", hr);  \
-     }                         \
+#define IFS(fn)                                                     \
+  do {                                                              \
+     if (SUCCEEDED(hr)) {                                           \
+        hr = (fn);                                                  \
+        if (FAILED(hr)) fprintf(stderr, #fn " failed %08x\n", hr);  \
+     }                                                              \
   } while (0)
 
+// modified version of DEFINE_GUID from guiddef.h.
+#define WEBP_DEFINE_GUID(name, l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
+  const GUID name = { l, w1, w2, { b1, b2,  b3,  b4,  b5,  b6,  b7,  b8 } }
+
 #ifdef __cplusplus
 #define MAKE_REFGUID(x) (x)
 #else
 #define MAKE_REFGUID(x) &(x)
 #endif
 
+typedef struct WICFormatImporter {
+  const GUID* pixel_format;
+  int bytes_per_pixel;
+  int (*import)(WebPPicture* const, const uint8_t* const, int);
+} WICFormatImporter;
+
 static HRESULT OpenInputStream(const char* filename, IStream** ppStream) {
   HRESULT hr = S_OK;
   IFS(SHCreateStreamOnFileA(filename, STGM_READ, ppStream));
   if (FAILED(hr))
-    printf("Error opening input file %s (%08x)\n", filename, hr);
+    fprintf(stderr, "Error opening input file %s (%08x)\n", filename, hr);
   return hr;
 }
 
 static HRESULT ReadPictureWithWIC(const char* filename,
                                   WebPPicture* const pic, int keep_alpha) {
+  // From Microsoft SDK 7.0a -- wincodec.h
+  // Create local copies for compatibility when building against earlier
+  // versions of the SDK.
+  WEBP_DEFINE_GUID(GUID_WICPixelFormat24bppBGR_,
+                   0x6fddc324, 0x4e03, 0x4bfe,
+                   0xb1, 0x85, 0x3d, 0x77, 0x76, 0x8d, 0xc9, 0x0c);
+  WEBP_DEFINE_GUID(GUID_WICPixelFormat24bppRGB_,
+                   0x6fddc324, 0x4e03, 0x4bfe,
+                   0xb1, 0x85, 0x3d, 0x77, 0x76, 0x8d, 0xc9, 0x0d);
+  WEBP_DEFINE_GUID(GUID_WICPixelFormat32bppBGRA_,
+                   0x6fddc324, 0x4e03, 0x4bfe,
+                   0xb1, 0x85, 0x3d, 0x77, 0x76, 0x8d, 0xc9, 0x0f);
+  WEBP_DEFINE_GUID(GUID_WICPixelFormat32bppRGBA_,
+                   0xf5c7ad2d, 0x6a8d, 0x43dd,
+                   0xa7, 0xa8, 0xa2, 0x99, 0x35, 0x26, 0x1a, 0xe9);
+  const WICFormatImporter alphaFormatImporters[] = {
+    { &GUID_WICPixelFormat32bppBGRA_, 4, WebPPictureImportBGRA },
+    { &GUID_WICPixelFormat32bppRGBA_, 4, WebPPictureImportRGBA },
+    { NULL, 0, NULL },
+  };
+  const WICFormatImporter nonAlphaFormatImporters[] = {
+    { &GUID_WICPixelFormat24bppBGR_, 3, WebPPictureImportBGR },
+    { &GUID_WICPixelFormat24bppRGB_, 3, WebPPictureImportRGB },
+    { NULL, 0, NULL },
+  };
   HRESULT hr = S_OK;
   IWICBitmapFrameDecode* pFrame = NULL;
   IWICFormatConverter* pConverter = NULL;
@@ -123,9 +161,10 @@ static HRESULT ReadPictureWithWIC(const char* filename,
   IWICBitmapDecoder* pDecoder = NULL;
   IStream* pStream = NULL;
   UINT frameCount = 0;
-  UINT width, height = 0;
+  UINT width = 0, height = 0;
   BYTE* rgb = NULL;
   WICPixelFormatGUID srcPixelFormat = { 0 };
+  const WICFormatImporter* importer = NULL;
   GUID srcContainerFormat = { 0 };
   const GUID* alphaContainers[] = {
     &GUID_ContainerFormatBmp,
@@ -140,9 +179,10 @@ static HRESULT ReadPictureWithWIC(const char* filename,
           CLSCTX_INPROC_SERVER, MAKE_REFGUID(IID_IWICImagingFactory),
           (LPVOID*)&pFactory));
   if (hr == REGDB_E_CLASSNOTREG) {
-    printf("Couldn't access Windows Imaging Component (are you running \n");
-    printf("Windows XP SP3 or newer?). Most formats not available.\n");
-    printf("Use -s for the available YUV input.\n");
+    fprintf(stderr,
+            "Couldn't access Windows Imaging Component (are you running "
+            "Windows XP SP3 or newer?). Most formats not available. "
+            "Use -s for the available YUV input.\n");
   }
   // Prepare for image decoding.
   IFS(OpenInputStream(filename, &pStream));
@@ -150,36 +190,52 @@ static HRESULT ReadPictureWithWIC(const char* filename,
           WICDecodeMetadataCacheOnDemand, &pDecoder));
   IFS(IWICBitmapDecoder_GetFrameCount(pDecoder, &frameCount));
   if (SUCCEEDED(hr) && frameCount == 0) {
-    printf("No frame found in input file.\n");
+    fprintf(stderr, "No frame found in input file.\n");
     hr = E_FAIL;
   }
   IFS(IWICBitmapDecoder_GetFrame(pDecoder, 0, &pFrame));
   IFS(IWICBitmapFrameDecode_GetPixelFormat(pFrame, &srcPixelFormat));
   IFS(IWICBitmapDecoder_GetContainerFormat(pDecoder, &srcContainerFormat));
 
-  has_alpha = keep_alpha;
-  for (i = 0;
-       has_alpha && i < sizeof(alphaContainers)/sizeof(alphaContainers[0]);
-       ++i) {
-    if (IsEqualGUID(&srcContainerFormat, alphaContainers[i])) {
-      has_alpha =
-          IsEqualGUID(&srcPixelFormat, &GUID_WICPixelFormat32bppRGBA) ||
-          IsEqualGUID(&srcPixelFormat, &GUID_WICPixelFormat32bppBGRA);
-      break;
+  if (keep_alpha) {
+    for (i = 0;
+         i < sizeof(alphaContainers) / sizeof(alphaContainers[0]);
+         ++i) {
+      if (IsEqualGUID(MAKE_REFGUID(srcContainerFormat),
+                      MAKE_REFGUID(*alphaContainers[i]))) {
+        has_alpha =
+            IsEqualGUID(MAKE_REFGUID(srcPixelFormat),
+                        MAKE_REFGUID(GUID_WICPixelFormat32bppRGBA_)) ||
+            IsEqualGUID(MAKE_REFGUID(srcPixelFormat),
+                        MAKE_REFGUID(GUID_WICPixelFormat32bppBGRA_));
+        break;
+      }
     }
   }
 
   // Prepare for pixel format conversion (if necessary).
   IFS(IWICImagingFactory_CreateFormatConverter(pFactory, &pConverter));
+
+  for (importer = has_alpha ? alphaFormatImporters : nonAlphaFormatImporters;
+       hr == S_OK && importer->import != NULL; ++importer) {
+    BOOL canConvert;
+    const HRESULT cchr = IWICFormatConverter_CanConvert(
+        pConverter,
+        MAKE_REFGUID(srcPixelFormat),
+        MAKE_REFGUID(*importer->pixel_format),
+        &canConvert);
+    if (SUCCEEDED(cchr) && canConvert) break;
+  }
+  if (importer->import == NULL) hr = E_FAIL;
+
   IFS(IWICFormatConverter_Initialize(pConverter, (IWICBitmapSource*)pFrame,
-          has_alpha ? MAKE_REFGUID(GUID_WICPixelFormat32bppRGBA)
-                    : MAKE_REFGUID(GUID_WICPixelFormat24bppRGB),
+          importer->pixel_format,
           WICBitmapDitherTypeNone,
           NULL, 0.0, WICBitmapPaletteTypeCustom));
 
   // Decode.
   IFS(IWICFormatConverter_GetSize(pConverter, &width, &height));
-  stride = (has_alpha ? 4 : 3) * width * sizeof(*rgb);
+  stride = importer->bytes_per_pixel * width * sizeof(*rgb);
   if (SUCCEEDED(hr)) {
     rgb = (BYTE*)malloc(stride * height);
     if (rgb == NULL)
@@ -191,18 +247,17 @@ static HRESULT ReadPictureWithWIC(const char* filename,
   // WebP conversion.
   if (SUCCEEDED(hr)) {
     int ok;
-#ifdef WEBP_EXPERIMENTAL_FEATURES
-    if (has_alpha) {
-      pic->colorspace |= WEBP_CSP_ALPHA_BIT;
-    }
-#endif
     pic->width = width;
     pic->height = height;
-    ok = has_alpha ? WebPPictureImportRGBA(pic, rgb, stride)
-                   : WebPPictureImportRGB(pic, rgb, stride);
+    ok = importer->import(pic, rgb, stride);
     if (!ok)
       hr = E_FAIL;
   }
+  if (SUCCEEDED(hr)) {
+    if (has_alpha && keep_alpha == 2) {
+      WebPCleanupTransparentArea(pic);
+    }
+  }
 
   // Cleanup.
   if (pConverter != NULL) IUnknown_Release(pConverter);
@@ -262,7 +317,7 @@ static int ReadJPEG(FILE* in_file, WebPPicture* const pic) {
   dinfo.err = jpeg_std_error(&jerr.pub);
   jerr.pub.error_exit = my_error_exit;
 
-  if (setjmp (jerr.setjmp_buffer)) {
+  if (setjmp(jerr.setjmp_buffer)) {
  Error:
     jpeg_destroy_decompress(&dinfo);
     goto End;
@@ -306,8 +361,8 @@ static int ReadJPEG(FILE* in_file, WebPPicture* const pic) {
     row_ptr += stride;
   }
 
-  jpeg_finish_decompress (&dinfo);
-  jpeg_destroy_decompress (&dinfo);
+  jpeg_finish_decompress(&dinfo);
+  jpeg_destroy_decompress(&dinfo);
 
   // WebP conversion.
   pic->width = width;
@@ -325,8 +380,8 @@ static int ReadJPEG(FILE* in_file, WebPPicture* const pic) {
 static int ReadJPEG(FILE* in_file, WebPPicture* const pic) {
   (void)in_file;
   (void)pic;
-  printf("JPEG support not compiled. Please install the libjpeg development "
-         "package before building.\n");
+  fprintf(stderr, "JPEG support not compiled. Please install the libjpeg "
+          "development package before building.\n");
   return 0;
 }
 #endif
@@ -358,7 +413,7 @@ static int ReadPNG(FILE* in_file, WebPPicture* const pic, int keep_alpha) {
   if (setjmp(png_jmpbuf(png))) {
  Error:
     png_destroy_read_struct(&png, NULL, NULL);
-    if (rgb) free(rgb);
+    free(rgb);
     goto End;
   }
 
@@ -374,7 +429,8 @@ static int ReadPNG(FILE* in_file, WebPPicture* const pic, int keep_alpha) {
   png_set_strip_16(png);
   png_set_packing(png);
   if (color_type == PNG_COLOR_TYPE_PALETTE) png_set_palette_to_rgb(png);
-  if (color_type == PNG_COLOR_TYPE_GRAY) {
+  if (color_type == PNG_COLOR_TYPE_GRAY ||
+      color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
     if (bit_depth < 8) {
       png_set_expand_gray_1_2_4_to_8(png);
     }
@@ -391,11 +447,6 @@ static int ReadPNG(FILE* in_file, WebPPicture* const pic, int keep_alpha) {
     png_set_strip_alpha(png);
     has_alpha = 0;
   }
-#ifdef WEBP_EXPERIMENTAL_FEATURES
-  if (has_alpha) {
-    pic->colorspace |= WEBP_CSP_ALPHA_BIT;
-  }
-#endif
 
   num_passes = png_set_interlace_handling(png);
   png_read_update_info(png, info);
@@ -417,6 +468,10 @@ static int ReadPNG(FILE* in_file, WebPPicture* const pic, int keep_alpha) {
                  : WebPPictureImportRGB(pic, rgb, stride);
   free(rgb);
 
+  if (ok && has_alpha && keep_alpha == 2) {
+    WebPCleanupTransparentArea(pic);
+  }
+
  End:
   return ok;
 }
@@ -425,16 +480,80 @@ static int ReadPNG(FILE* in_file, WebPPicture* const pic, int keep_alpha) {
   (void)in_file;
   (void)pic;
   (void)keep_alpha;
-  printf("PNG support not compiled. Please install the libpng development "
-         "package before building.\n");
+  fprintf(stderr, "PNG support not compiled. Please install the libpng "
+          "development package before building.\n");
+  return 0;
+}
+#endif
+
+#ifdef WEBP_HAVE_TIFF
+static int ReadTIFF(const char* const filename,
+                    WebPPicture* const pic, int keep_alpha) {
+  TIFF* const tif = TIFFOpen(filename, "r");
+  uint32 width, height;
+  uint32* raster;
+  int ok = 0;
+  int dircount = 1;
+
+  if (tif == NULL) {
+    fprintf(stderr, "Error! Cannot open TIFF file '%s'\n", filename);
+    return 0;
+  }
+
+  while (TIFFReadDirectory(tif)) ++dircount;
+
+  if (dircount > 1) {
+    fprintf(stderr, "Warning: multi-directory TIFF files are not supported.\n"
+                    "Only the first will be used, %d will be ignored.\n",
+                    dircount - 1);
+  }
+
+  TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &width);
+  TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &height);
+  raster = (uint32*)_TIFFmalloc(width * height * sizeof(*raster));
+  if (raster != NULL) {
+    if (TIFFReadRGBAImageOriented(tif, width, height, raster,
+                                  ORIENTATION_TOPLEFT, 1)) {
+      const int stride = width * sizeof(*raster);
+      pic->width = width;
+      pic->height = height;
+      // TIFF data is ABGR
+#ifdef __BIG_ENDIAN__
+      TIFFSwabArrayOfLong(raster, width * height);
+#endif
+      ok = keep_alpha
+         ? WebPPictureImportRGBA(pic, (const uint8_t*)raster, stride)
+         : WebPPictureImportRGBX(pic, (const uint8_t*)raster, stride);
+    }
+    _TIFFfree(raster);
+  } else {
+    fprintf(stderr, "Error allocating TIFF RGBA memory!\n");
+  }
+
+  if (ok && keep_alpha == 2) {
+    WebPCleanupTransparentArea(pic);
+  }
+
+  TIFFClose(tif);
+  return ok;
+}
+#else
+static int ReadTIFF(const char* const filename,
+                    WebPPicture* const pic, int keep_alpha) {
+  (void)filename;
+  (void)pic;
+  (void)keep_alpha;
+  fprintf(stderr, "TIFF support not compiled. Please install the libtiff "
+          "development package before building.\n");
   return 0;
 }
 #endif
 
 typedef enum {
-  PNG = 0,
-  JPEG,
-  UNSUPPORTED,
+  PNG_ = 0,
+  JPEG_,
+  TIFF_,  // 'TIFF' clashes with libtiff
+  UNSUPPORTED
 } InputFileFormat;
 
 static InputFileFormat GetImageType(FILE* in_file) {
@@ -449,9 +568,11 @@ static InputFileFormat GetImageType(FILE* in_file) {
 
   magic = (buf[0] << 24) | (buf[1] << 16) | (buf[2] << 8) | buf[3];
   if (magic == 0x89504E47U) {
-    format = PNG;
+    format = PNG_;
   } else if (magic >= 0xFFD8FF00U && magic <= 0xFFD8FFFFU) {
-    format = JPEG;
+    format = JPEG_;
+  } else if (magic == 0x49492A00 || magic == 0x4D4D002A) {
+    format = TIFF_;
   }
   return format;
 }
@@ -468,10 +589,12 @@ static int ReadPicture(const char* const filename, WebPPicture* const pic,
   if (pic->width == 0 || pic->height == 0) {
     // If no size specified, try to decode it as PNG/JPEG (as appropriate).
     const InputFileFormat format = GetImageType(in_file);
-    if (format == PNG) {
+    if (format == PNG_) {
       ok = ReadPNG(in_file, pic, keep_alpha);
-    } else if (format == JPEG) {
+    } else if (format == JPEG_) {
       ok = ReadJPEG(in_file, pic);
+    } else if (format == TIFF_) {
+      ok = ReadTIFF(filename, pic, keep_alpha);
     }
   } else {
     // If image size is specified, infer it as YUV format.
@@ -502,7 +625,7 @@ static void PrintByteCount(const int bytes[4], int total_size,
     total += bytes[s];
     if (totals) totals[s] += bytes[s];
   }
-  fprintf(stderr,"| %7d  (%.1f%%)\n", total, 100.f * total / total_size);
+  fprintf(stderr, "| %7d  (%.1f%%)\n", total, 100.f * total / total_size);
 }
 
 static void PrintPercents(const int counts[4], int total) {
@@ -510,7 +633,7 @@ static void PrintPercents(const int counts[4], int total) {
   for (s = 0; s < 4; ++s) {
     fprintf(stderr, "|      %2d%%", 100 * counts[s] / total);
   }
-  fprintf(stderr,"| %7d\n", total);
+  fprintf(stderr, "| %7d\n", total);
 }
 
 static void PrintValues(const int values[4]) {
@@ -518,20 +641,58 @@ static void PrintValues(const int values[4]) {
   for (s = 0; s < 4; ++s) {
     fprintf(stderr, "| %7d ", values[s]);
   }
-  fprintf(stderr,"|\n");
+  fprintf(stderr, "|\n");
 }
 
-static void PrintExtraInfo(const WebPPicture* const pic, int short_output) {
+static void PrintFullLosslessInfo(const WebPAuxStats* const stats,
+                                  const char* const description) {
+  fprintf(stderr, "Lossless-%s compressed size: %d bytes\n",
+          description, stats->lossless_size);
+  if (stats->lossless_features) {
+    fprintf(stderr, "  * Lossless features used:");
+    if (stats->lossless_features & 1) fprintf(stderr, " PREDICTION");
+    if (stats->lossless_features & 2) fprintf(stderr, " CROSS-COLOR-TRANSFORM");
+    if (stats->lossless_features & 4) fprintf(stderr, " SUBTRACT-GREEN");
+    if (stats->lossless_features & 8) fprintf(stderr, " PALETTE");
+    fprintf(stderr, "\n");
+  }
+  fprintf(stderr, "  * Precision Bits: histogram=%d transform=%d cache=%d\n",
+          stats->histogram_bits, stats->transform_bits, stats->cache_bits);
+  if (stats->palette_size > 0) {
+    fprintf(stderr, "  * Palette size:   %d\n", stats->palette_size);
+  }
+}
+
+static void PrintExtraInfoLossless(const WebPPicture* const pic,
+                                   int short_output,
+                                   const char* const file_name) {
   const WebPAuxStats* const stats = pic->stats;
   if (short_output) {
     fprintf(stderr, "%7d %2.2f\n", stats->coded_size, stats->PSNR[3]);
-  } else{
+  } else {
+    fprintf(stderr, "File:      %s\n", file_name);
+    fprintf(stderr, "Dimension: %d x %d\n", pic->width, pic->height);
+    fprintf(stderr, "Output:    %d bytes\n", stats->coded_size);
+    PrintFullLosslessInfo(stats, "ARGB");
+  }
+}
+
+static void PrintExtraInfoLossy(const WebPPicture* const pic, int short_output,
+                                const char* const file_name) {
+  const WebPAuxStats* const stats = pic->stats;
+  if (short_output) {
+    fprintf(stderr, "%7d %2.2f\n", stats->coded_size, stats->PSNR[3]);
+  } else {
     const int num_i4 = stats->block_count[0];
     const int num_i16 = stats->block_count[1];
     const int num_skip = stats->block_count[2];
     const int total = num_i4 + num_i16;
-    fprintf(stderr,
-            "%7d bytes Y-U-V-All-PSNR %2.2f %2.2f %2.2f   %2.2f dB\n",
+    fprintf(stderr, "File:      %s\n", file_name);
+    fprintf(stderr, "Dimension: %d x %d%s\n",
+            pic->width, pic->height,
+            stats->alpha_data_size ? " (with alpha)" : "");
+    fprintf(stderr, "Output:    "
+            "%d bytes Y-U-V-All-PSNR %2.2f %2.2f %2.2f   %2.2f dB\n",
             stats->coded_size,
             stats->PSNR[0], stats->PSNR[1], stats->PSNR[2], stats->PSNR[3]);
     if (total > 0) {
@@ -547,9 +708,9 @@ static void PrintExtraInfo(const WebPPicture* const pic, int short_output) {
               100.f * stats->header_bytes[0] / stats->coded_size,
               stats->header_bytes[1],
               100.f * stats->header_bytes[1] / stats->coded_size);
-      if (stats->alpha_data_size) {
-        fprintf(stderr, "             transparency:   %6d\n",
-                stats->alpha_data_size);
+      if (stats->alpha_data_size > 0) {
+        fprintf(stderr, "             transparency:   %6d (%.1f dB)\n",
+                stats->alpha_data_size, stats->PSNR[4]);
       }
       if (stats->layer_data_size) {
         fprintf(stderr, "             enhancement:    %6d\n",
@@ -575,8 +736,11 @@ static void PrintExtraInfo(const WebPPicture* const pic, int short_output) {
       fprintf(stderr, " segments total:  ");
       PrintByteCount(totals, stats->coded_size, NULL);
     }
+    if (stats->lossless_size > 0) {
+      PrintFullLosslessInfo(stats, "alpha");
+    }
   }
-  if (pic->extra_info) {
+  if (pic->extra_info != NULL) {
     const int mb_w = (pic->width + 15) / 16;
     const int mb_h = (pic->height + 15) / 16;
     const int type = pic->extra_info_type;
@@ -615,10 +779,11 @@ static int DumpPicture(const WebPPicture* const picture, const char* PGM_name) {
   const int uv_width = (picture->width + 1) / 2;
   const int uv_height = (picture->height + 1) / 2;
   const int stride = (picture->width + 1) & ~1;
-  const int alpha_height = picture->a ? picture->height : 0;
+  const int alpha_height =
+      WebPPictureHasTransparency(picture) ? picture->height : 0;
   const int height = picture->height + uv_height + alpha_height;
   FILE* const f = fopen(PGM_name, "wb");
-  if (!f) return 0;
+  if (f == NULL) return 0;
   fprintf(f, "P5\n%d %d\n255\n", stride, height);
   for (y = 0; y < picture->height; ++y) {
     if (fwrite(picture->y + y * picture->y_stride, picture->width, 1, f) != 1)
@@ -642,6 +807,15 @@ static int DumpPicture(const WebPPicture* const picture, const char* PGM_name) {
 
 //------------------------------------------------------------------------------
 
+static int ProgressReport(int percent, const WebPPicture* const picture) {
+  printf("[%s]: %3d %%      \r",
+         (char*)picture->user_data, percent);
+  fflush(stdout);
+  return 1;  // all ok
+}
+
+//------------------------------------------------------------------------------
+
 static void HelpShort(void) {
   printf("Usage:\n\n");
   printf("   cwebp [options] -q quality input.png -o output.webp\n\n");
@@ -654,7 +828,7 @@ static void HelpLong(void) {
   printf("Usage:\n");
   printf(" cwebp [-preset <...>] [options] in_file [-o out_file]\n\n");
   printf("If input size (-s) for an image is not specified, "
-         "it is assumed to be a PNG or JPEG file.\n");
+         "it is assumed to be a PNG, JPEG or TIFF file.\n");
 #ifdef HAVE_WINCODEC_H
   printf("Windows builds can take as input any of the files handled by WIC\n");
 #endif
@@ -662,6 +836,8 @@ static void HelpLong(void) {
   printf("  -h / -help  ............ short help\n");
   printf("  -H / -longhelp  ........ long help\n");
   printf("  -q <float> ............. quality factor (0:small..100:big)\n");
+  printf("  -alpha_q <int> ......... Transparency-compression quality "
+         "(0..100).\n");
   printf("  -preset <string> ....... Preset setting, one of:\n");
   printf("                            default, photo, picture,\n");
   printf("                            drawing, icon, text\n");
@@ -681,10 +857,6 @@ static void HelpLong(void) {
   printf("  -partition_limit <int> . limit quality to fit the 512k limit on\n");
   printf("                           "
          "the first partition (0=no degradation ... 100=full)\n");
-#ifdef WEBP_EXPERIMENTAL_FEATURES
-  printf("  -alpha_comp <int> ...... set the transparency-compression\n");
-  printf("  -noalpha ............... discard any transparency information.\n");
-#endif
   printf("  -pass <int> ............ analysis pass number (1..10)\n");
   printf("  -crop <x> <y> <w> <h> .. crop picture with the given rectangle\n");
   printf("  -resize <w> <h> ........ resize picture (after any cropping)\n");
@@ -692,7 +864,17 @@ static void HelpLong(void) {
   printf("  -444 / -422 / -gray ..... Change colorspace\n");
 #endif
   printf("  -map <int> ............. print map of extra info.\n");
+  printf("  -print_ssim ............ prints averaged SSIM distortion.\n");
+  printf("  -print_psnr ............ prints averaged PSNR distortion.\n");
   printf("  -d <file.pgm> .......... dump the compressed output (PGM file).\n");
+  printf("  -alpha_method <int> .... Transparency-compression method (0..1)\n");
+  printf("  -alpha_filter <string> . predictive filtering for alpha plane.\n");
+  printf("                           One of: none, fast (default) or best.\n");
+  printf("  -alpha_cleanup ......... Clean RGB values in transparent area.\n");
+  printf("  -noalpha ............... discard any transparency information.\n");
+  printf("  -lossless .............. Encode image losslessly.\n");
+  printf("  -hint <string> ......... Specify image characteristics hint.\n");
+  printf("                           One of: photo, picture or graph\n");
 
   printf("\n");
   printf("  -short ................. condense printed message\n");
@@ -703,6 +885,7 @@ static void HelpLong(void) {
 #endif
   printf("  -v ..................... verbose, e.g. print encoding/decoding "
          "times\n");
+  printf("  -progress .............. report encoding progress\n");
   printf("\n");
   printf("Experimental Options:\n");
   printf("  -af .................... auto-adjust filter strength.\n");
@@ -727,33 +910,36 @@ static const char* const kErrorMessages[] = {
   "header bits using -partition_limit. More details are available "
   "in the manual (`man cwebp`)",
   "PARTITION_OVERFLOW: Partition is too big to fit 16M",
-  "BAD_WRITE: Picture writer returned an I/O error"
-  "FILE_TOO_BIG: File would be too big to fit in 4G"
+  "BAD_WRITE: Picture writer returned an I/O error",
+  "FILE_TOO_BIG: File would be too big to fit in 4G",
+  "USER_ABORT: encoding abort requested by user"
 };
 
 //------------------------------------------------------------------------------
 
 int main(int argc, const char *argv[]) {
+  int return_value = -1;
   const char *in_file = NULL, *out_file = NULL, *dump_file = NULL;
   FILE *out = NULL;
   int c;
   int short_output = 0;
   int quiet = 0;
-  int keep_alpha = 0;
+  int keep_alpha = 1;
   int crop = 0, crop_x = 0, crop_y = 0, crop_w = 0, crop_h = 0;
   int resize_w = 0, resize_h = 0;
+  int show_progress = 0;
   WebPPicture picture;
+  int print_distortion = 0;        // 1=PSNR, 2=SSIM
+  WebPPicture original_picture;    // when PSNR or SSIM is requested
   WebPConfig config;
   WebPAuxStats stats;
   Stopwatch stop_watch;
 
-#ifdef WEBP_EXPERIMENTAL_FEATURES
-  keep_alpha = 1;
-#endif
-
-  if (!WebPPictureInit(&picture) || !WebPConfigInit(&config)) {
+  if (!WebPPictureInit(&picture) ||
+      !WebPPictureInit(&original_picture) ||
+      !WebPConfigInit(&config)) {
     fprintf(stderr, "Error! Version mismatch!\n");
-    goto Error;
+    return -1;
   }
 
   if (argc == 1) {
@@ -773,6 +959,12 @@ int main(int argc, const char *argv[]) {
     } else if (!strcmp(argv[c], "-d") && c < argc - 1) {
       dump_file = argv[++c];
       config.show_compressed = 1;
+    } else if (!strcmp(argv[c], "-print_ssim")) {
+      config.show_compressed = 1;
+      print_distortion = 2;
+    } else if (!strcmp(argv[c], "-print_psnr")) {
+      config.show_compressed = 1;
+      print_distortion = 1;
     } else if (!strcmp(argv[c], "-short")) {
       short_output++;
     } else if (!strcmp(argv[c], "-s") && c < argc - 2) {
@@ -782,6 +974,41 @@ int main(int argc, const char *argv[]) {
       config.method = strtol(argv[++c], NULL, 0);
     } else if (!strcmp(argv[c], "-q") && c < argc - 1) {
       config.quality = (float)strtod(argv[++c], NULL);
+    } else if (!strcmp(argv[c], "-alpha_q") && c < argc - 1) {
+      config.alpha_quality = strtol(argv[++c], NULL, 0);
+    } else if (!strcmp(argv[c], "-alpha_method") && c < argc - 1) {
+      config.alpha_compression = strtol(argv[++c], NULL, 0);
+    } else if (!strcmp(argv[c], "-alpha_cleanup")) {
+      keep_alpha = keep_alpha ? 2 : 0;
+    } else if (!strcmp(argv[c], "-alpha_filter") && c < argc - 1) {
+      ++c;
+      if (!strcmp(argv[c], "none")) {
+        config.alpha_filtering = 0;
+      } else if (!strcmp(argv[c], "fast")) {
+        config.alpha_filtering = 1;
+      } else if (!strcmp(argv[c], "best")) {
+        config.alpha_filtering = 2;
+      } else {
+        fprintf(stderr, "Error! Unrecognized alpha filter: %s\n", argv[c]);
+        goto Error;
+      }
+    } else if (!strcmp(argv[c], "-noalpha")) {
+      keep_alpha = 0;
+    } else if (!strcmp(argv[c], "-lossless")) {
+      config.lossless = 1;
+      picture.use_argb = 1;
+    } else if (!strcmp(argv[c], "-hint") && c < argc - 1) {
+      ++c;
+      if (!strcmp(argv[c], "photo")) {
+        config.image_hint = WEBP_HINT_PHOTO;
+      } else if (!strcmp(argv[c], "picture")) {
+        config.image_hint = WEBP_HINT_PICTURE;
+      } else if (!strcmp(argv[c], "graph")) {
+        config.image_hint = WEBP_HINT_GRAPH;
+      } else {
+        fprintf(stderr, "Error! Unrecognized image hint: %s\n", argv[c]);
+        goto Error;
+      }
     } else if (!strcmp(argv[c], "-size") && c < argc - 1) {
       config.target_size = strtol(argv[++c], NULL, 0);
     } else if (!strcmp(argv[c], "-psnr") && c < argc - 1) {
@@ -804,12 +1031,6 @@ int main(int argc, const char *argv[]) {
       config.segments = strtol(argv[++c], NULL, 0);
     } else if (!strcmp(argv[c], "-partition_limit") && c < argc - 1) {
       config.partition_limit = strtol(argv[++c], NULL, 0);
-#ifdef WEBP_EXPERIMENTAL_FEATURES
-    } else if (!strcmp(argv[c], "-alpha_comp") && c < argc - 1) {
-      config.alpha_compression = strtol(argv[++c], NULL, 0);
-    } else if (!strcmp(argv[c], "-noalpha")) {
-      keep_alpha = 0;
-#endif
     } else if (!strcmp(argv[c], "-map") && c < argc - 1) {
       picture.extra_info_type = strtol(argv[++c], NULL, 0);
 #ifdef WEBP_EXPERIMENTAL_FEATURES
@@ -838,6 +1059,8 @@ int main(int argc, const char *argv[]) {
       printf("%d.%d.%d\n",
         (version >> 16) & 0xff, (version >> 8) & 0xff, version & 0xff);
       return 0;
+    } else if (!strcmp(argv[c], "-progress")) {
+      show_progress = 1;
     } else if (!strcmp(argv[c], "-quiet")) {
       quiet = 1;
     } else if (!strcmp(argv[c], "-preset") && c < argc - 1) {
@@ -873,6 +1096,24 @@ int main(int argc, const char *argv[]) {
       in_file = argv[c];
     }
   }
+  if (in_file == NULL) {
+    fprintf(stderr, "No input file specified!\n");
+    HelpShort();
+    goto Error;
+  }
+
+  // Check for unsupported command line options for lossless mode and log
+  // warning for such options.
+  if (!quiet && config.lossless == 1) {
+    if (config.target_size > 0 || config.target_PSNR > 0) {
+      fprintf(stderr, "Encoding for specified size or PSNR is not supported"
+                      " for lossless encoding. Ignoring such option(s)!\n");
+    }
+    if (config.partition_limit > 0) {
+      fprintf(stderr, "Partition limit option is not required for lossless"
+                      " encoding. Ignoring this option!\n");
+    }
+  }
 
   if (!WebPValidateConfig(&config)) {
     fprintf(stderr, "Error! Invalid configuration.\n");
@@ -884,9 +1125,11 @@ int main(int argc, const char *argv[]) {
     StopwatchReadAndReset(&stop_watch);
   }
   if (!ReadPicture(in_file, &picture, keep_alpha)) {
-    fprintf(stderr, "Error! Cannot read input picture\n");
+    fprintf(stderr, "Error! Cannot read input picture file '%s'\n", in_file);
     goto Error;
   }
+  picture.progress_hook = (show_progress && !quiet) ? ProgressReport : NULL;
+
   if (verbose) {
     const double time = StopwatchReadAndReset(&stop_watch);
     fprintf(stderr, "Time to read input: %.3fs\n", time);
@@ -895,7 +1138,7 @@ int main(int argc, const char *argv[]) {
   // Open the output
   if (out_file) {
     out = fopen(out_file, "wb");
-    if (!out) {
+    if (out == NULL) {
       fprintf(stderr, "Error! Cannot open output file '%s'\n", out_file);
       goto Error;
     } else {
@@ -912,15 +1155,21 @@ int main(int argc, const char *argv[]) {
       fprintf(stderr, "be performed, but its results discarded.\n\n");
     }
   }
-  picture.stats = &stats;
+  if (!quiet) {
+    picture.stats = &stats;
+    picture.user_data = (void*)in_file;
+  }
 
   // Compress
   if (verbose) {
     StopwatchReadAndReset(&stop_watch);
   }
-  if (crop != 0 && !WebPPictureCrop(&picture, crop_x, crop_y, crop_w, crop_h)) {
-    fprintf(stderr, "Error! Cannot crop picture\n");
-    goto Error;
+  if (crop != 0) {
+    // We use self-cropping using a view.
+    if (!WebPPictureView(&picture, crop_x, crop_y, crop_w, crop_h, &picture)) {
+      fprintf(stderr, "Error! Cannot crop picture\n");
+      goto Error;
+    }
   }
   if ((resize_w | resize_h) > 0) {
     if (!WebPPictureRescale(&picture, resize_w, resize_h)) {
@@ -931,6 +1180,9 @@ int main(int argc, const char *argv[]) {
   if (picture.extra_info_type > 0) {
     AllocExtraInfo(&picture);
   }
+  if (print_distortion > 0) {  // Save original picture for later comparison
+    WebPPictureCopy(&picture, &original_picture);
+  }
   if (!WebPEncode(&config, &picture)) {
     fprintf(stderr, "Error! Cannot encode picture as WebP\n");
     fprintf(stderr, "Error code: %d (%s)\n",
@@ -944,20 +1196,39 @@ int main(int argc, const char *argv[]) {
 
   // Write info
   if (dump_file) {
-    DumpPicture(&picture, dump_file);
+    if (picture.use_argb) {
+      fprintf(stderr, "Warning: can't dump file (-d option) in lossless mode.");
+    } else if (!DumpPicture(&picture, dump_file)) {
+      fprintf(stderr, "Warning, couldn't dump picture %s\n", dump_file);
+    }
   }
+
   if (!quiet) {
-    PrintExtraInfo(&picture, short_output);
+    if (config.lossless) {
+      PrintExtraInfoLossless(&picture, short_output, in_file);
+    } else {
+      PrintExtraInfoLossy(&picture, short_output, in_file);
+    }
   }
+  if (!quiet && !short_output && print_distortion > 0) {  // print distortion
+    float values[5];
+    WebPPictureDistortion(&picture, &original_picture,
+                          (print_distortion == 1) ? 0 : 1, values);
+    fprintf(stderr, "%s: Y:%.2f U:%.2f V:%.2f A:%.2f  Total:%.2f\n",
+            (print_distortion == 1) ? "PSNR" : "SSIM",
+            values[0], values[1], values[2], values[3], values[4]);
+  }
+  return_value = 0;
 
  Error:
   free(picture.extra_info);
   WebPPictureFree(&picture);
+  WebPPictureFree(&original_picture);
   if (out != NULL) {
     fclose(out);
   }
 
-  return 0;
+  return return_value;
 }
 
 //------------------------------------------------------------------------------
